'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

/**
 * Errors.
 *
 * @type {Object}
 */

var ERRORS = {

  /**
   * Client.
   */

  client_disconnected: {
    message: 'The Nexus client disconnected from the server.',
    data: []
  },

  /**
   * Server.
   */

  server_error: {
    message: '{error.message}',
    data: ['error']
  },

  /**
   * Messages.
   */

  message_invalid: {
    message: 'Encountered an invalid message object.',
    data: ['message']
  },

  message_property_invalid: {
    message: 'Encountered a message with an invalid `{property}` property.',
    data: ['message', 'property']
  },

  /**
   * Operations.
   */

  operation_duplicate: {
    message: 'A duplicate operation was submitted to the server.',
    data: ['operation']
  },

  operation_invalid: {
    message: 'Encountered an invalid operation object.',
    data: ['operation']
  },

  operation_property_invalid: {
    message: 'Encountered an operation with an invalid `{property}` property.',
    data: ['operation', 'property']
  },

  operation_submit_failed: {
    message: 'An operation failed be submitted after the maximum number of retries was reached.',
    data: ['operation']
  },

  operation_version_mismatch: {
    message: 'An operation was submitted with a version that could not be resolved against the latest snapshot.',
    data: ['operation']
  },

  /**
   * Pubsub.
   */

  pubsub_id_invalid: {
    message: 'The `id` emitted by the pubsub adapter was invalid. It should be an ID string.',
    data: ['value']
  },

  pubsub_operation_invalid: {
    message: 'The `operation` emitted by the pubsub adapter was invalid. It should be an operation object.',
    data: ['value']
  },

  /**
   * Snapshots.
   */

  snapshot_invalid: {
    message: 'Encountered an invalid snapshot object.',
    data: ['snapshot']
  },

  snapshot_property_invalid: {
    message: 'Encountered a snapshot with an invalid `{property}` property.',
    data: ['snapshot', 'property']
  },

  /**
   * Stores.
   */

  store_commit_invalid: {
    message: 'The `commit()` method of the store adapter must return a boolean indicating whether the commit succeeded.',
    data: []
  },

  /**
   * Types.
   */

  type_invalid: {
    message: 'Encountered an invalid OT type "{type}".',
    data: ['type']
  },

  type_unknown: {
    message: 'Encountered an unknown OT type "{type}".',
    data: ['type']
  },

  /**
   * OT.
   */

  apply_version_mismatch: {
    message: 'An OT `type.apply()` was attempted with an operation and snapshot with non-matching versions.',
    data: ['operation', 'snapshot']
  },

  transform_version_mismatch: {
    message: 'An OT `type.transform()` was attempted with two operations with non-matching versions.',
    data: ['operation', 'snapshot']
  }

  /**
   * Export.
   *
   * @type {Object}
   */

};exports.default = ERRORS;