'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _hat = require('hat');

var _hat2 = _interopRequireDefault(_hat);

var _omit = require('lodash/omit');

var _omit2 = _interopRequireDefault(_omit);

var _checkers = require('../utils/checkers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Debug.
 *
 * @type {Function}
 */

var debug = (0, _debug2.default)('nexus:connection');

/**
 * Connection.
 *
 * @type {Connection}
 */

var Connection = function (_Emitter) {
  _inherits(Connection, _Emitter);

  /**
   * Constructor.
   *
   * @param {Object} config
   */

  function Connection() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Connection);

    var _this = _possibleConstructorReturn(this, (Connection.__proto__ || Object.getPrototypeOf(Connection)).call(this));

    var server = config.server,
        transport = config.transport,
        context = config.context;

    _this.server = server;
    _this.transport = transport;
    _this.context = context;
    _this.id = (0, _hat2.default)();
    _this.subscriptions = {};

    transport.on('readable', function () {
      var data = _this.transport.read();
      if (data === null) return _this.close();
      _this.onMessage(data);
    });
    return _this;
  }

  /**
   * Close the connection.
   */

  _createClass(Connection, [{
    key: 'close',
    value: function close() {
      var subscriptions = this.subscriptions;


      for (var key in subscriptions) {
        var sub = subscriptions[key];
        sub.push(null);
      }

      this.emit('close');
    }

    /**
     * Send a `message` to the client.
     *
     * @param {Object} message
     */

  }, {
    key: 'send',
    value: function send(message) {
      debug('send', this.id, { message: message });
      this.transport.write(message);
    }

    /**
     * Send a connect message to the client for a `document`.
     *
     * @param {String} document
     */

  }, {
    key: 'sendConnect',
    value: function sendConnect(document) {
      this.send({
        action: 'connect',
        id: this.id
      });
    }

    /**
     * Send a `snapshot` message to the client for a `document` and `action`.
     *
     * @param {String} document
     * @param {String} action
     * @param {Object} snapshot
     */

  }, {
    key: 'sendSnapshot',
    value: function sendSnapshot(document, action, snapshot) {
      var message = {
        action: action,
        document: document
      };

      if (snapshot) {
        message.snapshot = snapshot;
      }

      this.send(message);
    }

    /**
     * Send an `operation` message to the client for a `document` and `action`.
     *
     * @param {String} document
     * @param {String} action
     * @param {Object} operation
     */

  }, {
    key: 'sendOperation',
    value: function sendOperation(document, action, operation) {
      var message = {
        action: action,
        document: document,
        id: operation.id,
        version: operation.version
      };

      if (operation.data !== undefined) {
        message.data = operation.data;
      }

      this.send(message);
    }

    /**
     * Send `operations` to the server for a `document`.
     *
     * @param {String} document
     * @param {String} action
     * @param {Array} operations
     */

  }, {
    key: 'sendOperations',
    value: function sendOperations(document, action, operations) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = operations[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var op = _step.value;

          this.sendOperation(document, action, op);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }

    /**
     * Send an unsubscribe message to the client for a `document`.
     *
     * @param {String} document
     */

  }, {
    key: 'sendUnsubscribe',
    value: function sendUnsubscribe() {
      this.send({
        action: 'unsubscribe',
        document: document
      });
    }

    /**
     * On message.
     *
     * @param {Object} message
     * @return {Mixed}
     */

  }, {
    key: 'onMessage',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(message) {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                debug('onMessage', this.id, { message: message });

                (0, _checkers.checkMessage)(message);

                _context.prev = 2;
                _context.t0 = message.action;
                _context.next = _context.t0 === 'connect' ? 6 : _context.t0 === 'fetch' ? 9 : _context.t0 === 'submit' ? 12 : _context.t0 === 'subscribe' ? 15 : _context.t0 === 'unsubscribe' ? 18 : 21;
                break;

              case 6:
                _context.next = 8;
                return this.onMessageConnect(message);

              case 8:
                return _context.abrupt('return', _context.sent);

              case 9:
                _context.next = 11;
                return this.onMessageFetch(message);

              case 11:
                return _context.abrupt('return', _context.sent);

              case 12:
                _context.next = 14;
                return this.onMessageSubmit(message);

              case 14:
                return _context.abrupt('return', _context.sent);

              case 15:
                _context.next = 17;
                return this.onMessageSubscribe(message);

              case 17:
                return _context.abrupt('return', _context.sent);

              case 18:
                _context.next = 20;
                return this.onMessageUnsubscribe(message);

              case 20:
                return _context.abrupt('return', _context.sent);

              case 21:
                _context.next = 28;
                break;

              case 23:
                _context.prev = 23;
                _context.t1 = _context['catch'](2);

                this.emit('error', _context.t1);
                this.server.emit('error', _context.t1);

                this.send(_extends({}, message, {
                  error: {
                    code: _context.t1.code,
                    message: _context.t1.message,
                    data: _context.t1.data
                  }
                }));

              case 28:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[2, 23]]);
      }));

      function onMessage(_x2) {
        return _ref.apply(this, arguments);
      }

      return onMessage;
    }()

    /**
     * On "connect" message.
     *
     * @param {Object} message
     * @return {Object}
     */

  }, {
    key: 'onMessageConnect',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(message) {
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                this.sendConnect();

              case 1:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function onMessageConnect(_x3) {
        return _ref2.apply(this, arguments);
      }

      return onMessageConnect;
    }()

    /**
     * On "fetch" message.
     *
     * @param {Object} message
     * @return {Object|Void}
     */

  }, {
    key: 'onMessageFetch',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(message) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        var _options$action, action, context, server, document, version, ops, snapshot;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _options$action = options.action, action = _options$action === undefined ? 'fetch' : _options$action;
                context = this.context, server = this.server;
                document = message.document, version = message.version;
                ops = [];
                snapshot = void 0;

                if (!(version == null)) {
                  _context3.next = 11;
                  break;
                }

                _context3.next = 8;
                return server.findSnapshot(document, { context: context });

              case 8:
                snapshot = _context3.sent;
                _context3.next = 14;
                break;

              case 11:
                _context3.next = 13;
                return server.listOperations(document, version, undefined, { context: context });

              case 13:
                ops = _context3.sent;

              case 14:

                this.sendOperations(document, 'broadcast', ops);
                this.sendSnapshot(document, action, snapshot);

              case 16:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function onMessageFetch(_x4) {
        return _ref3.apply(this, arguments);
      }

      return onMessageFetch;
    }()

    /**
     * On "submit" message.
     *
     * @param {Object} message
     * @return {Object}
     */

  }, {
    key: 'onMessageSubmit',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(message) {
        var context, server, document, ops, ack;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                context = this.context, server = this.server;
                document = message.document;
                ops = void 0;
                _context4.prev = 3;
                _context4.next = 6;
                return server.submit(document, message, { context: context });

              case 6:
                ops = _context4.sent;
                _context4.next = 13;
                break;

              case 9:
                _context4.prev = 9;
                _context4.t0 = _context4['catch'](3);

                if (!(_context4.t0.code !== 'operation_duplicate')) {
                  _context4.next = 13;
                  break;
                }

                throw _context4.t0;

              case 13:

                if (ops) {
                  this.sendOperations(document, 'broadcast', ops);
                }

                ack = (0, _omit2.default)(message, 'data');

                this.sendOperation(document, 'submit', ack);

              case 16:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[3, 9]]);
      }));

      function onMessageSubmit(_x6) {
        return _ref4.apply(this, arguments);
      }

      return onMessageSubmit;
    }()

    /**
     * On "subscribe" message.
     *
     * @param {Object} message
     * @return {Object|Void}
     */

  }, {
    key: 'onMessageSubscribe',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(message) {
        var _this2 = this;

        var context, server, subscriptions, id, document, version, subscription, previous;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                context = this.context, server = this.server, subscriptions = this.subscriptions, id = this.id;
                document = message.document, version = message.version;
                _context6.next = 4;
                return server.subscribe(document, version, { context: context });

              case 4:
                subscription = _context6.sent;
                previous = subscriptions[document];


                if (previous) {
                  previous.push(null);
                }

                subscriptions[document] = subscription;
                subscription.on('readable', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
                  var op;
                  return regeneratorRuntime.wrap(function _callee5$(_context5) {
                    while (1) {
                      switch (_context5.prev = _context5.next) {
                        case 0:
                          op = subscription.read();

                          if (!(op === null)) {
                            _context5.next = 5;
                            break;
                          }

                          delete subscriptions[document];
                          subscription.emit('close');
                          return _context5.abrupt('return');

                        case 5:
                          if (!op.id.startsWith(id + ':')) {
                            _context5.next = 7;
                            break;
                          }

                          return _context5.abrupt('return');

                        case 7:

                          // Otherwise, broadcast the op to the client.
                          _this2.sendOperation(document, 'broadcast', op);

                        case 8:
                        case 'end':
                          return _context5.stop();
                      }
                    }
                  }, _callee5, _this2);
                })));

                // Call into the fetch handler to fetch the current snapshot or operations.
                _context6.next = 11;
                return this.onMessageFetch(message, { action: 'subscribe' });

              case 11:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function onMessageSubscribe(_x7) {
        return _ref5.apply(this, arguments);
      }

      return onMessageSubscribe;
    }()

    /**
     * On "unsubscribe" message.
     *
     * @param {Object} message
     */

  }, {
    key: 'onMessageUnsubscribe',
    value: function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(message) {
        var document, subscriptions, sub;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                document = message.document, subscriptions = message.subscriptions;
                sub = subscriptions[document];


                if (sub) {
                  sub.push(null);
                }

                this.sendUnsubscribe(document);

              case 4:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function onMessageUnsubscribe(_x8) {
        return _ref7.apply(this, arguments);
      }

      return onMessageUnsubscribe;
    }()
  }]);

  return Connection;
}(_events2.default);

/**
 * Export.
 *
 * @type {Connection}
 */

exports.default = Connection;