'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _template = require('lodash/template');

var _template2 = _interopRequireDefault(_template);

var _errors = require('../constants/errors');

var _errors2 = _interopRequireDefault(_errors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Nexus error.
 *
 * @type {NexusError}
 */

var NexusError = function (_Error) {
  _inherits(NexusError, _Error);

  /**
   * Create a Nexus error by `code` (or message) with `data`.
   *
   * @type {String} code|message
   * @param {Object} data
   */

  function NexusError(code) {
    var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, NexusError);

    var obj = _errors2.default[code];
    var message = void 0;

    if (!obj) {
      message = 'An unrecognized Nexus error was thrown! The code "' + code + '" is not a valid Nexus error code.';
    } else if (obj.message == null) {
      message = 'An invalid Nexus error was defined! The "' + code + '" error has no message definition.';
    } else if (obj.data == null) {
      message = 'An invalid Nexus error was defined! The "' + code + '" error has no property definition.';
    } else {
      var fn = (0, _template2.default)(obj.message, { interpolate: /{([\s\S]+?)}/g });
      message = code + ' - ' + fn(data);

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = obj.data[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _key = _step.value;

          if (!(_key in data)) {
            message = 'An invalid Nexus error was thrown! The `' + _key + '` error property was not provided when a "' + code + '" error was thrown.';
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      for (var key in data) {
        if (!obj.data.includes(key)) {
          message = 'An invalid Nexus error was thrown! The `' + key + '` error property is not one of the recognized properties of "' + code + '" errors.';
        }
      }
    }

    var _this = _possibleConstructorReturn(this, (NexusError.__proto__ || Object.getPrototypeOf(NexusError)).call(this, message));

    _this.name = 'NexusError';
    _this.code = code;
    _this.data = data;
    return _this;
  }

  return NexusError;
}(Error);

/**
 * Export.
 *
 * @type {NexusError}
 */

exports.default = NexusError;